﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.SimulationCore.AgentBased;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Common;

namespace HeuristicLab.SimulationCore.Samples {
  [Item("GameOfLifeAgent", "A game of life agent.")]
  [StorableClass]
  public class GameOfLifeAgent: Item, IAgent {
    [Storable]
    private bool alive;
    public bool Alive {
      get {
        return alive;
      }
      set {
        alive = value;
      }
    }

    [Storable]
    private bool _alive;

    [Storable]
    private GameOfLifeSimulation simulation;
    public GameOfLifeSimulation Simulation {
      get {
        return simulation;
      }
    }

    public GameOfLifeAgent(GameOfLifeSimulation simulation) : base() {
      this.simulation = simulation;
      alive = _alive = false;
    }
    [StorableConstructor]
    private GameOfLifeAgent(bool deserializing) : base(deserializing) { }
    private GameOfLifeAgent(GameOfLifeAgent original, Cloner cloner)
      : base(original, cloner) {
      alive = original.alive;
      _alive = original._alive;
      simulation = original.simulation;
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new GameOfLifeAgent(this, cloner);
    }

    #region IAgent Members
    public void InitStep() {
    }

    public void PerformStep() {
      int aliveCount = 0;
      foreach (GameOfLifeAgent neighbor in simulation.GetNeighbors(this)) {
        if (neighbor.Alive)
          aliveCount++;
      }

      if (!alive && aliveCount == 3)
        _alive = true;
      else if (alive && (aliveCount < 2 || aliveCount > 3))
        _alive = false;
    }

    public void FinalizeStep() {
      alive = _alive;
    }
    #endregion
  }
}
